@@ -22,7 +22,7 @@ from scipy.special import logsumexp
 from scipy.stats import poisson
 from mpmath import polylog
 
-import midr.c_archimedean as c_arch
+import c_archimedean as c_arch
 
 
 def lsum(x_values, is_log=True, axis=0):
@@ -32,8 +32,8 @@ def lsum(x_values, is_log=True, axis=0):
     :param is_log:
     :param axis:
     :return:
-    >>> lsum(np.array([1, 2, 3, 4]))
-    4.440189698561196
+    # >>> lsum(np.array([1, 2, 3, 4]))
+    # 4.440189698561196
     >>> lsum(np.array([[0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
     ...    [0.79410993, 0.76175687, 0.0709562],
@@ -60,20 +60,7 @@ def lsum(x_values, is_log=True, axis=0):
     array([1.67247612, 1.32058311, 1.69157834, 1.50086587, 1.479448  ,
            1.62823149, 1.73370009, 1.75351973, 1.43774153, 1.67670621])
     """
-    if is_log:
-        b_i = x_values
-    else:
-        b_i = np.log(x_values)
-    b_max = np.amax(b_i, axis=axis)
-    if axis == 1:
-        return b_max + logsumexp(
-            b_i - b_max.reshape(-1, 1),
-            axis=axis
-        )
-    return b_max + logsumexp(
-        b_i - b_max,
-        axis=axis
-    )
+    return c_arch.lsum(x_values, axis, is_log)
 
 
 def lssum(x_values, x_sign=np.nan, is_log=True):
@@ -99,16 +86,7 @@ def lssum(x_values, x_sign=np.nan, is_log=True):
     array([1.0729724 , 0.48860043, 0.13450368, 0.28073852, 0.85281759,
            0.77384662, 1.1716088 , 0.18345907, 0.6112606 , 0.29341583])
     """
-    if is_log:
-        b_i_sign = x_sign
-        b_i = x_values
-    else:
-        b_i_sign = np.sign(x_values)
-        b_i = np.log(abs(x_values))
-    b_i_sign = b_i_sign.reshape(-1, 1)
-    b_max = np.amax(b_i, axis=0)
-    res = np.sum(b_i_sign * np.exp(b_i - b_max), axis=0)
-    return b_max + np.log(res)
+    return c_arch.llsum(x_values, signff(1/3.2, np.array([1, 2, 3]), 3), True)
 
 
 def signff(alpha, j, d):
@@ -128,22 +106,7 @@ def signff(alpha, j, d):
     assert 0.0 < alpha, "alpha value:" + str(alpha)
     assert alpha <= 1.0, "alpha value:" + str(alpha)
     assert d >= 0.0, "d value:" + str(d)
-    res = np.zeros(shape=j.shape[0])
-    if alpha == 1.0:
-        for i in range(j.shape[0]):
-            if j[i] == int(d):
-                res[i] = 1.0
-            else:
-                res[i] = (-1.0) ** (d - j[i])
-    else:
-        for i in range(j.shape[0]):
-            if j[i] > d:
-                res[i] = np.nan
-            else:
-                x = alpha * j[i]
-                if x != np.floor(x):
-                    res[i] = (-1.0) ** (j[i] - np.ceil(x))
-    return res
+    return c_arch.signff(alpha, j, d)
 
 
 def log1mexpunit(x):
@@ -151,7 +114,10 @@ def log1mexpunit(x):
     compute log(1-exp(-a)
     :param x:
     :return:
+    >>> log1mexpunit(2.0)
+    -0.14541345786885906
     """
+    return c_arch.log1mexp(x);
     if x <= np.log(2.0):
         return np.log(-np.expm1(-x))
     else:
@@ -163,7 +129,10 @@ def log1mexpvec(x):
     compute log(1-exp(-a)
     :param x:
     :return:
+    >>> log1mexpvec(np.array([2.0, 3.0, 4.0]))
+    array([-0.14541346, -0.05106918, -0.01848545])
     """
+    return np.asarray(c_arch.log1mexpvec(x))
     res = np.empty_like(x)
     eps = np.log(2.0)
     with np.errstate(invalid='ignore'):
@@ -358,11 +327,7 @@ def ipsi_clayton(u_values, theta, is_log=False):
            [-1.05452015, -0.85128374, -2.14210478],
            [-2.47465594, -2.58334647, -1.40228078]])
     """
-    x = np.array(u_values, dtype=np.float64)
-    res = np.sign(theta) * ((x ** -theta) - 1.0)
-    if is_log:
-        return np.array(np.log(res), dtype=np.float64)
-    return np.array(res, dtype=np.float64)
+    return c_arch.ipsi_clayton(u_values, theta, is_log)
 
 
 def psi_clayton(u_values, theta):
@@ -530,6 +495,22 @@ def pdf_clayton(u_values, theta, is_log=False):
     ...    [0.22437343, 0.16907646, 0.5740400],
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
+    ...    -0.2,
+    ...    is_log=True)
+    array([ 0.64386088, -0.09219481,  0.87324848,  0.90978732,  0.81025042,
+            0.453309  ,  0.53709342,  0.3984068 ,  0.66544891,  0.42404594])
+    >>> pdf_clayton(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
     ...    0.2,
     ...    is_log=True)
     array([-0.12264018,  0.13487358, -0.40809375, -0.4061165 , -0.39266393,
@@ -550,6 +531,7 @@ def pdf_clayton(u_values, theta, is_log=False):
     array([0.88458189, 1.1443921 , 0.66491654, 0.66623254, 0.67525564,
            1.0480272 , 0.89668514, 1.00407535, 1.007351  , 1.03652896])
     """
+    return c_arch.pdf_clayton(u_values, theta, is_log)
     d = float(u_values.shape[1])
     lu = np.sum(np.log(u_values), axis=1)
     t_var = np.sum(ipsi_clayton(u_values, theta), axis=1)
@@ -959,22 +941,20 @@ def eulerian_all(n):
            1.310354e+06, 1.310354e+06, 4.551920e+05, 4.784000e+04,
            1.013000e+03, 1.000000e+00])
     """
-    return c_arch.eulerian_all(n)
+    return np.asarray(c_arch.eulerian_all(n))
 
 
-def polyneval(coef, x):
+def polyneval(coef, x, is_log_z=False):
     """
-
     :param coef:
     :param x:
     :return:
-    >>> polyneval(eulerian_all(10), np.array([-4, -3]))
-    array([1.12058925e+08, 9.69548800e+06])
+    >>> polyneval(np.array(eulerian_all(10)), np.array([4, 3]))
+    array([4.42208200e+09, 8.80405504e+08])
+    >>> polyneval(np.array(eulerian_all(10)), np.log(np.array([4, 3])), True)
+    array([4.42208200e+09, 8.80405504e+08])
     """
-    return np.polyval(
-        p=np.tile(coef.reshape((-1, 1)), (1, len(x))),
-        x=x
-    )
+    return c_arch.polyneval(coef, x, is_log_z)
 
 
 def polylog(z, s, is_log_z=True):
@@ -983,21 +963,13 @@ def polylog(z, s, is_log_z=True):
     :param s:
     :param is_log_z:
     :return:
-    >>> polylog(np.array([0.01556112, 0.00108968, 0.00889932]), -2)
-    array([-4.10048812, -6.81751128, -4.68610298])
+    >>> polylog(np.array([0.01556112, 0.00108968, 0.00889932]), -2, False)
+    array([-4.1004881 , -6.81751129, -4.68610299])
     >>> polylog(np.log(np.array([0.01556112, 0.00108968, 0.00889932])), -2,
     ... True)
-    array([-4.10048812, -6.81751128, -4.68610298])
+    array([-4.1004881 , -6.81751129, -4.68610299])
     """
-    n = -int(s)
-    if is_log_z:
-        w = z
-        z = np.exp(w)
-        return np.log(c_arch.polyneval(eulerian_all(n), z)) + \
-               w - np.array(n + 1.0) * np.array(c_arch.log1mexpvec(-w))
-    else:
-        return np.log(c_arch.polyneval(eulerian_all(n), z)) + np.log(z) \
-               - np.array(n + 1.0) * np.array(np.log1p(-z))
+    return c_arch.polylog(z, int(s), is_log_z)
 
 
 def pdf_frank(u_values, theta, is_log=False):
@@ -1020,8 +992,8 @@ def pdf_frank(u_values, theta, is_log=False):
     ...    [0.66752741, 0.69487362, 0.3329266]
     ...    ]),
     ...    5.0)
-    array([0.1523755 , 1.83967836, 0.02685253, 0.15136271, 0.18938126,
-           1.44811357, 0.0905827 , 0.21205361, 1.08804124, 0.74757725])
+    array([0.1523755 , 1.83967837, 0.02685253, 0.1513627 , 0.18938125,
+           1.44811361, 0.09058269, 0.21205362, 1.08804125, 0.74757727])
     >>> pdf_frank(np.array([
     ...    [0.42873569, 0.18285458, 0.9514195],
     ...    [0.25148149, 0.05617784, 0.3378213],
@@ -1036,23 +1008,10 @@ def pdf_frank(u_values, theta, is_log=False):
     ...    ]),
     ...    5.0,
     ...    is_log=True)
-    array([-1.88140742,  0.60959075, -3.61739525, -1.8880763 , -1.66399306,
-            0.37026172, -2.40149208, -1.55091616,  0.08437905, -0.29091763])
+    array([-1.88140742,  0.60959076, -3.6173953 , -1.88807634, -1.66399309,
+            0.37026175, -2.40149211, -1.55091612,  0.08437906, -0.29091761])
     """
-    assert not np.isnan(theta), "pdf_frank: theta is nan"
-    if theta == 0.0:
-        copula = np.zeros(shape=u_values.shape[0])
-    else:
-        d = float(u_values.shape[1])
-        usum = np.sum(u_values, axis=1)
-        lp = log1mexp(theta)
-        lpu = log1mexp(theta * u_values)
-        lu = np.sum(lpu, axis=1)
-        li = polylog(lp + np.sum(lpu - lp, axis=1), -(d - 1.0))
-        copula = (d - 1.0) * np.log(theta) + li - theta * usum - lu
-    if is_log:
-        return copula
-    return np.exp(copula)
+    return c_arch.pdf_frank(u_values, theta, is_log)
 
 
 def ipsi_gumbel(u_values, theta, is_log=False):
@@ -1109,9 +1068,7 @@ def ipsi_gumbel(u_values, theta, is_log=False):
            [ 0.48210469,  0.69018481, -0.70642309],
            [-1.08708931, -1.21263832,  0.11419028]])
     """
-    if is_log:
-        return theta * np.log(-np.log(u_values))
-    return (-np.log(u_values)) ** theta
+    return c_arch.ipsi_gumbel(u_values, theta, is_log)
 
 
 def psi_gumbel(u_values, theta):
@@ -1260,31 +1217,7 @@ def log_polyg(lx_var, alpha_var, d_var):
     array([ 0.35110025,  1.31419104,  1.07707314,  1.68854151,  1.80435943,
            -0.43406987,  0.23166651, -0.18316099,  0.62329368, -0.35013782])
     """
-    k = np.linspace(start=1.0, stop=d_var, num=int(d_var))
-    x = np.exp(lx_var)
-    lppois = np.zeros(shape=[int(d_var), int(lx_var.shape[0])])
-    for i in range(int(d_var)):
-        lppois[i] = poisson.logcdf(d_var - k[i], x)
-    llx = np.dot(k.reshape(-1, 1), lx_var.reshape(1, -1))
-    labspoch = np.zeros(shape=int(d_var))
-    for i in range(int(d_var)):
-        with np.errstate(divide='ignore'):
-            labspoch[i] = np.sum(
-                np.log(
-                    abs(alpha_var * float(i + 1) - (k - 1.0))
-                ),
-                axis=0
-            )
-    lfac = np.log(factorial(k))
-    lxabs = llx + lppois + np.tile(
-        labspoch - lfac, int(lx_var.shape[0])
-    ).reshape((int(d_var), int(lx_var.shape[0])), order='F') + np.repeat(
-        x, int(d_var)
-    ).reshape((int(d_var), int(lx_var.shape[0])), order='F')
-    return lssum(
-        x_values=lxabs,
-        x_sign=signff(alpha_var, k, d_var),
-    )
+    return c_arch.log_polyg(lx_var, alpha_var, d_var)
 
 
 def pdf_gumbel(u_values, theta, is_log=False):
@@ -1342,23 +1275,4 @@ def pdf_gumbel(u_values, theta, is_log=False):
     array([-0.06797204,  0.11058147,  1.26625419, -2.13290342, -1.75890767,
            -0.4483621 , -5.15135971, -2.95009109,  0.94790989,  1.04000301])
     """
-    d = float(u_values.shape[1])
-    mlu = -np.log(u_values)
-    lmlu = np.log(mlu)
-    lip = ipsi_gumbel(u_values, theta, is_log=True)
-    lnt = lsum(np.transpose(lip))
-    alpha = 1.0 / theta
-    assert 0.0 < alpha, "pdf_gumbel: 0.0 < alpha: alpha = " + str(alpha) + \
-                        ", theta = " + str(theta)
-    assert alpha <= 1.0, "pdf_gumbel: alpha <= 1.0: alpha = " + str(alpha) + \
-                         " theta = " + str(theta)
-    lx = alpha * lnt
-    ls = log_polyg(lx, alpha, d) - d * lx / alpha
-    lnc = -np.exp(lx)
-    dcopula = lnc + d * np.log(theta) + np.sum(
-        (theta - 1.0) * lmlu + mlu,
-        axis=1
-    ) + ls
-    if is_log:
-        return dcopula
-    return np.exp(dcopula)
+    return c_arch.pdf_gumbel(u_values, theta, is_log)
